from django.contrib.auth.models import User
from django.test import TestCase
from rest_framework import status
from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient


class CloudinaryTestCase(TestCase):
    ENDPOINT = '/auth/login'

    def setUp(self) -> None:
        super().setUp()

        self.user = User.objects.create_user('test', password='test123')
        self.admin_user = User.objects.create_superuser('testadmin', password='test123',
                                                        is_active=True)
        Token(user=self.admin_user).save()
        Token(user=self.user).save()

    def get_client(self, user, http_authorization_prefix='Token'):
        c = APIClient()
        c.credentials(HTTP_AUTHORIZATION='%s %s' % (http_authorization_prefix, user.auth_token.key))
        return c

    def test_token(self):
        data = {'username': 'test', 'password': 'test123'}

        resp = self.client.post(self.ENDPOINT, data=data)
        self.assertIn('token', resp.data, resp.content)

    def test_masquerade(self):
        data = {'user_id': self.user.id}
        client = self.get_client(self.admin_user)

        resp = client.post('/auth/masquerade', data=data)

        self.assertIn('token', resp.data, resp.content)

    def test_masquerade_when_non_superuser_tries(self):
        data = {'user_id': self.admin_user.id}
        client = self.get_client(self.user)

        resp = client.post('/auth/masquerade', data=data)

        self.assertEqual(resp.status_code, status.HTTP_403_FORBIDDEN, resp.content)



from rest_framework.authtoken.serializers import AuthTokenSerializer


class ExAuthTokenSerializer(AuthTokenSerializer):
    def validate_username(self, value):
        return value.lower() if value else value



from rest_framework.permissions import BasePermission


class IsSuperuser(BasePermission):
    """
    Allows access only to admin users.
    """

    def has_permission(self, request, view):
        return bool(request.user and request.user.is_superuser)



import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404
from django.utils.module_loading import import_string
from rest_framework.authtoken.models import Token
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.decorators import api_view, permission_classes
from rest_framework.response import Response
from rest_framework.serializers import ModelSerializer
from rest_framework.settings import api_settings

from .permissions import IsSuperuser
from .serializers import ExAuthTokenSerializer

L = logging.getLogger(__name__)


class ExObtainAuthToken(ObtainAuthToken):
    """
    To obtain authentication token for users using their ``username`` and ``password``
    Required fields: ``username``, ``password``
    """
    serializer_class = ExAuthTokenSerializer
    renderer_classes = api_settings.DEFAULT_RENDERER_CLASSES

    def post(self, request, *args, **kwargs):
        serializer = self.serializer_class(data=request.data,
                                           context={'request': request})
        serializer.is_valid(raise_exception=True)
        user = serializer.validated_data['user']
        token, created = Token.objects.get_or_create(user=user)
        serializer = self.get_serializer_class()(user)
        data = serializer.data
        data['token'] = token.key

        return Response(data)

    def get_serializer_class(self):
        user_model = get_user_model()

        serializer_str = getattr(settings, 'AUTH_SERIALIZER', None)
        try:
            return import_string(serializer_str)
        except (ImportError, AttributeError):
            class Default(ModelSerializer):
                class Meta:
                    model = user_model
                    exclude = ('password',)

            return Default


@api_view(['POST'])
@permission_classes((IsSuperuser,))
def masquerade_login(request, *args, **kwargs):
    user_id = request.data.get('user_id')

    try:
        user = get_user_model().objects.get(id=user_id)
    except (ObjectDoesNotExist, ValueError):
        raise Http404

    token, created = Token.objects.get_or_create(user=user)
    data = dict(token=token.key)

    return Response(data)


#: To bind CustomObtainAuthToken to system's URL
obtain_auth_token = ExObtainAuthToken.as_view()
